@@ -58,18 +58,27 @@ reg_def["CDR2"] = list(range(56, 65 + 1))
 reg_def["CDR3"] = list(range(105, 117 + 1))
 reg_def["CDR_all"] = reg_def["CDR1"] + reg_def["CDR2"] + reg_def["CDR3"]
 
-cdr3_mask = np.array([int(canonical_numbering[i][:-1]) in reg_def["CDR3"] for i in range(len(canonical_numbering))], dtype = np.int8)
+cdr3_mask = np.array([int(canonical_numbering[i][:-1]) in reg_def["CDR3"] for i in range(len(canonical_numbering))], dtype = np.uint8)
 all_cdrs_mask = np.array(
-    [int(canonical_numbering[i][:-1]) in reg_def["CDR_all"] for i in range(len(canonical_numbering))], dtype = np.int8)
+    [int(canonical_numbering[i][:-1]) in reg_def["CDR_all"] for i in range(len(canonical_numbering))], dtype = np.uint8)
 
 
 def get_region_mask(region):
-    """
-    Get mask for a specific region.
-    """
+    """Retrieve mask for a specific region.
+    
+    Parameters
+    ----------
+    region : list
+        List of unique positions from the canonical alignment to not mask
+
+    Returns
+    -------
+    numpy array
+        Specific region mask
+    """     
     
     if region == 'whole':
-        return np.ones(200, dtype = np.int8)
+        return np.ones(200, dtype = np.uint8)
     elif region == 'cdrs':
         return all_cdrs_mask
     elif region == 'cdr3':
@@ -77,12 +86,21 @@ def get_region_mask(region):
     else:
         assert np.all([i in canonical_numbering for i in region]), "At least one position in the user defined region, is not part of the 200 used positions. \
         Please remove such positions."
-        return np.array([i in region for i in canonical_numbering], dtype = np.int8)
+        return np.array([i in region for i in canonical_numbering], dtype = np.uint8)
 
 
 def canonical_alignment(anarci_output):
-    """
-    Alignment of anarci output
+    """Alignment of ANARCI output.
+    
+    Parameters
+    ----------
+    anarci_output : dict
+        ANARCI numbering of a sequence
+
+    Returns
+    -------
+    numpy array
+        Canonical alignment of a sequence
     """
 
     sequence = np.zeros(canonical_numbering_len, np.int8)
@@ -97,11 +115,22 @@ def canonical_alignment(anarci_output):
     sequence[(sequence == 45)] = 0
     return sequence
 
+
 oas_numbering_finder = re.compile('\d+.\'\: \'[A-Z]')
 def canonical_alignment_oas(anarci_output):
+    """OAS derived ANARCI output.
+    
+    Parameters
+    ----------
+    anarci_output : dict
+        OAS derived ANARCI numbering of a sequence
+
+    Returns
+    -------
+    numpy array
+        Canonical alignment of a sequence
     """
-    OAS has a different anarci output.
-    """
+    
     sequence = np.zeros(canonical_numbering_len, np.int8)
 
     for res in oas_numbering_finder.findall(anarci_output):
@@ -110,4 +139,4 @@ def canonical_alignment_oas(anarci_output):
         sequence[i] = int.from_bytes(seq.encode(), 'big')
 
     sequence[(sequence == 45)] = 0
-    return sequence
+    return 
\ No newline at end of file
